{ Demo program for ChroMathsome - Linear Algebra.

  This demonstration program shows how easy we can expand ChroMathsome - Linear Algebra.
  Note: many more examples of expanding ChroMathsome - Linear Algebra are given in the help files
  and other demonstration programs (e.g. the statistics example in \LinAlgP\Stat).
  Also note that many of these examples may be implemented in ChroMathsome - Linear Algebra
  in future versions (it's obvious that the code must be more streamlined and
  checked with many more example matrices before they can become a member of the
  excellent ChroMathsome - Linear Algebra family of functions ... ;-))
  When studying the code below, notice that writing new functions is, almost, nothing
  more than translating pseudo code (another pro for (and reason for you to use)
  ChroMathsome - Linear Algebra!).

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers (matrices and vectors)
  get freed, even when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), you can give most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!

  Read the help file for information about the implemented functions
  and their limitations/possibilities!
  Also, we've assumed the reader is familiar with these "new" functions.
  If not, consult an appropriate textbook.

  Author: John Merrit (john.merrit@net4all.be)
  Release date: June 30, 1998
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  ChroMathsome - Linear Algebra 1996-98 John Merrit
  All Rights Reserved

  Brought to you by the letter "E".
  We assume that the universe is expanding but we know ChroMathsome - Linear Algebra really is
  expandable!    }
unit Expan1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls, LinAlg, LinAlgP;

type
  TForm1 = class(TForm)
    Memo1: TRichEdit;
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    Version1: TMenuItem;
    Exit1: TMenuItem;
    PseudoInverse1: TMenuItem;
    Nullspace1: TMenuItem;
    DiagonalizeMatrix1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure Version1Click(Sender: TObject);
    procedure PseudoInverse1Click(Sender: TObject);
    procedure Nullspace1Click(Sender: TObject);
    procedure DiagonalizeMatrix1Click(Sender: TObject);
  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
   
    procedure PseudoInverseComplexMatrix(var M: ComplexMatrix);
    procedure RealMatrixNullSpace(var M: RealMatrix; var Empty: boolean);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-17 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,8,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Write a complex matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-17 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-17 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;


procedure TForm1.Exit1Click(Sender: TObject);
begin
     Close;
end;

{ Shows information about ChroMathsome version we're using.
  If you contact me (via e-mail), please specify this version information.
  This way, I know which version you're using at the moment.                }
procedure TForm1.Version1Click(Sender: TObject);
begin
     ChroMathsomeVersion;
end;

(* Find the pseudo-inverse (PI) of a (m*n)-matrix.
   If m = n -> (normal) inverse of matrix.
   Scheme: PI(M) = Inverse(M * M) * M with M the hermite-conjugate of M.
   Can be used to solve a system (with more equations than variables) to find the
   least-squares solution.   *)
procedure TForm1.PseudoInverseComplexMatrix(var M: ComplexMatrix);
var M1, M2: ComplexMatrix;
begin
     InitComplexMatrix(M1, 1, 1);
     InitComplexMatrix(M2, 1, 1);

     try

     Copy_ComplexMatrix(M1, M);
     ComplexMatrix_HermiteConjugate(M1);
     ComplexMatrix_Multiplication(M2, M1, M);
     ComplexMatrix_Inverse(M2);
     ComplexMatrix_Multiplication(M, M2, M1);

     finally
     DestroyComplexMatrix(M1);
     DestroyComplexMatrix(M2);
     end;
end;

{ Test the above. }
procedure TForm1.PseudoInverse1Click(Sender: TObject);
var M: ComplexMatrix;
begin
     InitComplexMatrix(M, 5, 5);

     try

     Random_ComplexMatrix(M, 10);
     Memo1.Lines.Add('Random matrix: ');
     Write_ComplexMatrix(M);

     PseudoInverseComplexMatrix(M);
     Memo1.Lines.Add('Pseudo-inverse matrix: ');
     Write_ComplexMatrix(M);

     ComplexMatrix_Inverse(M);
     Memo1.Lines.Add('Test -> inverse of pseudo-inverse matrix: ');
     Write_ComplexMatrix(M);
     
     finally
     DestroyComplexMatrix(M);
     end;
end;

(* Returns the nullspace for matrix M (more correctly: a base for the nullspace).
   If the nullspace is empty, Empty is true.
   The nullspace (aka. kernel) of a matrix is defined as the set of
   (non-nil) vectors X so that A * X = 0.
   An important application is the search for eigenvectors (eigenspace) for a certain
   eigenvalue of a matrix (see following procedure TForm1.Diagon...; ).
   Recall that  is an eigenvalue of a matrix A with associated eigenvector X if: A * X =  * X.
   To find the vector X, one must find the nullspace of (A -  * In) so that (A -  * In) * X = 0.
   Can also be used to solve systems of linear equations.                          *)
procedure TForm1.RealMatrixNullSpace(var M: RealMatrix; var Empty: boolean);
var i, j, k, l: integer;
    M1: RealMatrix;
begin
     InitRealMatrix(M1, 1, 1);
     try

     RealMatrix_Transpose(M);
     k:= M.Col;
     l:= M.Row;
     RealAugmentedMatrix(M); // M == M | In; In = identity matrix
     RealMatrix_HermiteNormalForm(M); // reduce to hermite form

     // number of nil rows of M is dimension of nullspace
     RealSubMatrix(M1, M, 1, M.Row, 1, k); // copy the In part to another matrix
     i:= RealMatrix_NumberOfNilRows(M1);

     if i = 0 then
     begin
       Empty:= true;
       Redimension_RealMatrix(M, M.Row, 1);
       NilRealMatrix(M);
     end
     else
     begin
       Empty:= false;
       // Get base vectors for nullspace.
       RealSubMatrix(M1, M, M.Row - i + 1, M.Row, k + 1, M.Col);
       Copy_RealMatrix(M, M1);
       RealMatrix_Transpose(M); // Columns are a base for the nullspace.
     end;

     finally
     DestroyRealMatrix(M1);
     end;
end;


(* Test. Note that we also test if A * X is indeed 0 by calculating the norm of vector A * X.  *)
procedure TForm1.Nullspace1Click(Sender: TObject);
var i: integer;
    M, M1, M2, M3: RealMatrix;
    V: RealVector;
    IsEmpty: boolean;
begin
      InitRealMatrix(M, 3, 3);

      InitRealMatrix(M1, 1, 1);
      InitRealMatrix(M2, 1, 1);
      InitRealMatrix(M3, 1, 1);
      InitRealVector(V, 1);
      try

      M.Matrix^[1]^[1]:=  1;
      M.Matrix^[1]^[2]:=  2;
      M.Matrix^[1]^[3]:= 2;
      
      M.Matrix^[2]^[1]:= 2;
      M.Matrix^[2]^[2]:= 0;
      M.Matrix^[2]^[3]:= 0;
      
      M.Matrix^[3]^[1]:= 0;
      M.Matrix^[3]^[2]:= -1;
      M.Matrix^[3]^[3]:= -1;
     
      Copy_RealMatrix(M1, M);
      RealMatrixNullSpace(M1, IsEmpty);

      // test
      // Check if A * X is indeed equal to zero for every base vector.
      if IsEmpty = false then
      begin
      Write_Message('Base for nullspace: ');
      Write_RealMatrix(M1);
      for i:= 1 to M1.Col do
      begin
      RealMatrixColToRealVector(V, M1, i);
      RealVectorToRealMatrix(M2, V);
      RealMatrix_Transpose(M2);
      RealMatrix_Multiplication(M3, M, M2);
      Write_Message('Test: Norm of vector: ' + FloatToStr(RealMatrix_Norm(M3)));
      end;
      end
      else Write_Message('Nullspace is empty!');

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealMatrix(M2);
      DestroyRealMatrix(M3);
      DestroyRealVector(V);
      end;
end;

{ Diagonalize a square matrix.
  The scheme is: MD = Inverse(Q) * M * Q
  where
    MD = diagonal matrix (matrix main diagonal elements are eigenvalues of matrix)
    M = original matrix
    Q = matrix where every column (v1 v2 ... vn) is an eigenvector for an eigenvalue so that
    Q = [ v1 v2 v3 ... vn ]
    If an eigenvalue has a multiplicity > 1, all eigenvectors belonging to this eigenvalue
    will be present in Q.
    The following examples will clearify this.

  Example 1:
  Matrix M has two eigenvalues namely 5 and 3
  where the eigenvalue 3 has a multiplicity of 2 (5 a multiplicity of 1).
  To create matrix Q, we find an eigenvector for eigenvalue 5 (=> v1)
  and, thus, two eigenvectors for eigenvalue 3 (=> v2 and v3)
  This results in matrix Q = [ v1 v2 v3 ].

  Example 2:
  ...
  
  More correctly: we find a base for the eigenspace belonging to the eigenvalue.
  Note: the eigenvectors do not need to be orthonormal (for this diagonalization scheme!).
  Also, note that we did not check if the original matrix may indeed be diagonalized!!!
  In this demo, we also ignored to check if the eigenvalue is complex or real.
  (that's your part!)                                                                     }
procedure TForm1.DiagonalizeMatrix1Click(Sender: TObject);
var i, j: integer;
    M, M1, M2, M3: RealMatrix;
    V: ComplexVector;
    V1: RealVector;
    IsEmpty, IsDoubleEigenvalue: boolean;
    c: extended;
begin
      ShowMessage('Memory consumed by matrices: ' + IntToStr(CurrentMatrixMemoryUse) + ' bytes.');

      InitRealMatrix(M, 3, 3);
      InitRealMatrix(M1, 1, 1);
      InitRealMatrix(M2, 1, 1);
      InitRealMatrix(M3, 1, 1);
      InitRealVector(V1, 1);
      InitComplexVector(V, 1);
      try

      M.Matrix^[1]^[1]:= 4;
      M.Matrix^[1]^[2]:= 1;
      M.Matrix^[1]^[3]:= -1;

      M.Matrix^[2]^[1]:= 2;
      M.Matrix^[2]^[2]:= 5;
      M.Matrix^[2]^[3]:= -2;

      M.Matrix^[3]^[1]:= 1;
      M.Matrix^[3]^[2]:= 1;
      M.Matrix^[3]^[3]:= 2;

      Write_Message('Example 1');
      Write_Message('Original matrix ');
      Write_RealMatrix(M);

      Copy_RealMatrix(M1, M); // We need the original matrix later ...

      // Find eigenvalues of the matrix.
      RealMatrix_Eigenvalues(V, M1);
      Write_Message('Matrix eigenvalues: ');
      for i:= 1 to V.Dimension do
      Write_Message(FloatToStr(V.Vector^[i].re));

      { Create matrix Q.
        Note: check for double eigenvalues!!! }
      for i:= 1 to V.Dimension do
      begin
      Copy_RealMatrix(M1, M);
      Write_Message('Characteristic matrix for eigenvalue ' + FloatToStr(V.Vector^[i].re) + ': ' );
      c:= V.Vector^[i].re;
      RealCharacteristicMatrix(M1, c);
      Write_RealMatrix(M1);
      { Base for eigenspace for eigenvalue ... }
      Write_Message('Base for nullspace for eigenvalue ' + FloatToStr(V.Vector^[i].re) + ':');
      RealMatrixNullSpace(M1, IsEmpty); // We ignore to check if IsEmpty is true ...
      Write_RealMatrix(M1);

      if i = 1 then Copy_RealMatrix(M2, M1) // add eigenvector(s) for first eigenvalue
      else
      begin // check for double eigenvalues!
         IsDoubleEigenValue:= false;
         for j:= 1 to i - 1 do
         if V.Vector^[j].re = c then IsDoubleEigenValue:= true;
         // if not double, add eigenvectors to Q (= M2)
         if IsDoubleEigenValue = false then // add eigenvectors for other eigenvalue(s)
         begin
         for j:= 1 to M1.Col do
         begin
         RealMatrixColToRealVector(V1, M1, j);
         RealMatrix_AddColumn(M2, V1);
         end;
         end;
      end;
      end;

      // Now, we have matrix Q.
      Write_Message('Matrix Q:');
      Write_RealMatrix(M2);
      Copy_RealMatrix(M3, M2);
      RealMatrix_Inverse(M3);
      Write_Message('Matrix Inverse(Q):');
      Write_RealMatrix(M3);

      // Test if Inverse(Q) * M * Q is indeed a diagonal matrix containing eigenvalues of original matrix.
      RealMatrix_Multiplication(M1, M3, M); // M1 = Inverse(Q) * M
      RealMatrix_Multiplication(M3, M1, M2); // M3 = M1 * Q
      Write_Message('Matrix Inverse(Q) * M * Q:');
      Write_RealMatrix(M3);

      { Example 2 }
      M.Matrix^[1]^[1]:= 1;
      M.Matrix^[1]^[2]:= -3;
      M.Matrix^[1]^[3]:= 3;

      M.Matrix^[2]^[1]:= 3;
      M.Matrix^[2]^[2]:= -5;
      M.Matrix^[2]^[3]:= 3;

      M.Matrix^[3]^[1]:= 6;
      M.Matrix^[3]^[2]:= -6;
      M.Matrix^[3]^[3]:= 4;

      Write_Message('Example 2');
      Write_Message('Original matrix ');
      Write_RealMatrix(M);

      Copy_RealMatrix(M1, M);

      RealMatrix_Eigenvalues(V, M1);
      Write_Message('Matrix eigenvalues: ');
      for i:= 1 to V.Dimension do
      Write_Message(FloatToStr(V.Vector^[i].re));

      { Create matrix Q.
        Note: check for double eigenvalues!!! }
      for i:= 1 to V.Dimension do
      begin
      Copy_RealMatrix(M1, M);
      Write_Message('Characteristic matrix for eigenvalue ' + FloatToStr(V.Vector^[i].re) + ': ' );
      c:= V.Vector^[i].re;
      RealCharacteristicMatrix(M1, c);
      Write_RealMatrix(M1);
      { Base for eigenspace for eigenvalue ... }
      Write_Message('Base for nullspace for eigenvalue ' + FloatToStr(V.Vector^[i].re) + ':');
      RealMatrixNullSpace(M1, IsEmpty);
      Write_RealMatrix(M1);

      if i = 1 then Copy_RealMatrix(M2, M1)
      else
      begin // check for double eigenvalues!
         IsDoubleEigenValue:= false;
         for j:= 1 to i - 1 do
         if V.Vector^[j].re = c then IsDoubleEigenValue:= true;
         // if not double, add eigenvectors to Q (= M2)
         if IsDoubleEigenValue = false then
         begin
         for j:= 1 to M1.Col do
         begin
         RealMatrixColToRealVector(V1, M1, j);
         RealMatrix_AddColumn(M2, V1);
         end;
         end;
      end;
      end;

      Write_Message('Matrix Q:');
      Write_RealMatrix(M2);
      Copy_RealMatrix(M3, M2);
      RealMatrix_Inverse(M3);
      Write_Message('Matrix Inverse(Q):');
      Write_RealMatrix(M3);

      // Test if Inverse(Q) * M * Q is indeed a diagonal matrix containing eigenvalues of original matrix.
      RealMatrix_Multiplication(M1, M3, M); // M1 = Inverse(Q) * M
      RealMatrix_Multiplication(M3, M1, M2); // M3 = M1 * Q
      Write_Message('Matrix Inverse(Q) * M * Q:');
      Write_RealMatrix(M3);

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealMatrix(M2);
      DestroyRealMatrix(M3);
      DestroyRealVector(V1);
      DestroyComplexVector(V);
      ShowMessage('Memory consumed by matrices: ' + IntToStr(CurrentMatrixMemoryUse) + ' bytes.');
      end;
end;

end.
