unit MatrxDlg;

interface

uses WinTypes, WinProcs, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, Dialogs, Messages;

type
  TEnterMatrixDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    MatrixGrid: TStringGrid;
    Edit1: TEdit;
    Edit2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    procedure Edit1Change(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure MatrixGridSetEditText(Sender: TObject; ACol, ARow: Longint;
      const Value: String);
   procedure FormClose(Sender: TObject; var Action: TCloseAction);

  private
    { Private declarations }

    procedure UpdateMatrixGrid;

  public
    { Public declarations }
  end;

var
  EnterMatrixDlg: TEnterMatrixDlg;

implementation

{$R *.DFM}

{ Check if users enters an integer value in the edit box! }
procedure TEnterMatrixDlg.Edit1Change(Sender: TObject);
var i: integer;
    ErrorOc: boolean;
begin
      ErrorOc:= false;
      if Edit1.Text <> '' then
      begin
      try
      i:= StrToInt(Edit1.Text);
      Except on EConvertError do ErrorOc:= true;
      end;
      if ((ErrorOc = true) or (i <= 0)) then
      begin
      MessageDlg('''' + Edit1.Text + '''' + ' is not a valid number of rows!', mtError,[mbOk], 0);
      Edit1.Text:= '3';
      end
      else
      begin
      MatrixGrid.RowCount:= i + 1;
      UpdateMatrixGrid;
      end;
      end;
end;

procedure TEnterMatrixDlg.Edit2Change(Sender: TObject);
var i: integer;
    ErrorOc: boolean;
begin
      ErrorOc:= false;
      if Edit2.Text <> '' then
      begin
      try
      i:= StrToInt(Edit2.Text);
      Except on EConvertError do ErrorOc:= true;
      end;
      if ((ErrorOc = true) or (i <= 0)) then
      begin
      MessageDlg('''' + Edit2.Text + '''' + ' is not a valid number of columns!', mtError,[mbOk], 0);
      Edit2.Text:= '3';
      end
      else
      begin
      MatrixGrid.ColCount:= i + 1;
      UpdateMatrixGrid;
      end;
      end;
end;

{ If numbers of rows/columns changes, update grid to reflect changes. }
procedure TEnterMatrixDlg.UpdateMatrixGrid;
var i, j: integer;
begin
     with MatrixGrid do
     begin
     for i:= 1 to ColCount - 1 do
     for j:= 1 to RowCount - 1 do
     Cells[i,j]:= '0';
     for j:= 1 to RowCount - 1 do
     Cells[0,j]:= 'Row ' + IntToStr(j);
     for j:= 1 to ColCount - 1 do
     Cells[j,0]:= 'Col ' + IntToStr(j);
     end;
end;

procedure TEnterMatrixDlg.FormActivate(Sender: TObject);
begin
     UpdateMatrixGrid;
     MatrixGrid.Options:= [goEditing, goAlwaysShowEditor, goVertLine, goHorzLine, goTabs];
end;

{ When user edits cells, make sure (s)he enters a floating point value! }
procedure TEnterMatrixDlg.MatrixGridSetEditText(Sender: TObject; ACol,
  ARow: Longint; const Value: String);
var i: extended;
begin
     if Value <> '' then
     begin
     try
     i:= StrToFloat(Value);
     except on EConvertError do
     begin
     MessageDlg('''' + Value + '''' + ' is not a valid floating point number!', mtError,[mbOk], 0);
     MatrixGrid.Cells[ACol,ARow]:= '0';
     end;
     end;
     end;
end;

{ Make sure both edit controls contain a valid integer when form closes. }
procedure TEnterMatrixDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
var I, J: integer;
begin
     if Edit1.Text = '' then Edit1.Text:= '1';
     if Edit2.Text = '' then Edit2.Text:= '1';
     with MatrixGrid do
     begin
     for I:= 1 to ColCount - 1 do
     for J:= 1 to RowCount - 1 do
     if Cells[I,J] = '' then Cells[I,J]:= '0';
     end;

end;


end.
