{ Demonstration program for ChroMathsome - Linear Algebra.

  This demo shows how to iterate through a matrix i.e. applying a mathematical function on all elements of a row/column
  in a matrix.

  Read the help file for information about the implemented functions
  and their limitations/possibilities!!!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: June 30, 1998
  Reader may do with this code whatever he/she wants.

  ChroMathsome - Linear Algebra 1996-98 John Merrit

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "E".
  "Progress has no patience - but something's got to give"              }

unit iter;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls, LinAlg, ExtCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    Real1: TMenuItem;
    IterateMatrix1: TMenuItem;
    Complex1: TMenuItem;
    IterateMatrix2: TMenuItem;
    Graph1: TMenuItem;
    Draw1: TMenuItem;
    PaintBox1: TPaintBox;
    procedure IterateMatrix1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure IterateMatrix2Click(Sender: TObject);
    procedure Draw1Click(Sender: TObject);
    
  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-17 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,8,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-17 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-17 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
     Close;
end;

{ Iterate matrix per row.
  We will replace the elements on a row by applying a mathematical function upon the elements. }
procedure TForm1.IterateMatrix1Click(Sender: TObject);
var M1: RealMatrix;
begin
     InitRealMatrix(M1,5, 5);
     ShowMessage('Memory consumed: ' + IntToStr(CurrentMatrixMemoryUse));
     Memo1.Visible:= true;
     PaintBox1.Visible:= false;

     try
     Random_RealMatrix(M1, 100);
     Write_Message('Original matrix:');
     Write_RealMatrix(M1);
     Write_Message('');

     // Replace every element of row 1 by the sine of the element
     RealMatrix_IterateRow(M1, 1, 'sin(x)');
     Write_RealMatrix(M1);
     Write_Message('');

     // Replace every element of row 2 ...
     RealMatrix_IterateRow(M1, 2, 'cos(log(x + 100)) + 25*sqrt(x)');
     Write_RealMatrix(M1);
     Write_Message('');

     // Replace every element of row 3 ...
     RealMatrix_IterateRow(M1, 3, 'ln(abs(x))');
     Write_RealMatrix(M1);
     Write_Message('');

     // Replace every element of row 4 ...
     RealMatrix_IterateRow(M1, 4, 'sqr(x)');
     Write_RealMatrix(M1);
     Write_Message('');

     // Replace every element of row 5 by the value 666
     RealMatrix_IterateRow(M1, 5, '666');
     Write_RealMatrix(M1);
     Write_Message('');

     // Applying a mathematical function on 1 matrix element.
     RealMatrix_ApplyFunctionOnMatrixElement(M1, 1, 1, 'sqrt(abs(x))');
     Write_RealMatrix(M1);
     Write_Message('');	

     finally
     DestroyRealMatrix(M1);
     ShowMessage('Memory consumed: ' + IntToStr(CurrentMatrixMemoryUse));
     end;
end;

{ Iterate matrix per column.
  We will replace the elements in a column by applying a mathematical function upon the elements. }
procedure TForm1.IterateMatrix2Click(Sender: TObject);
var M1: ComplexMatrix;
begin
     InitComplexMatrix(M1,5, 5);
     ShowMessage('Memory consumed: ' + IntToStr(CurrentMatrixMemoryUse));
     Memo1.Visible:= true;
     PaintBox1.Visible:= false;

     try
     Random_ComplexMatrix(M1, 100);
     Write_Message('Original matrix:');
     Write_ComplexMatrix(M1);
     Write_Message('');

     // Same as above but now per column and on a complex matrix.
     ComplexMatrix_IterateColumn(M1, 1, 'sin(x)');
     Write_ComplexMatrix(M1);
     Write_Message('');

     ComplexMatrix_IterateColumn(M1, 2, 'cos(log(x + 100)) + 25*sqrt(x)');
     Write_ComplexMatrix(M1);
     Write_Message('');

     ComplexMatrix_IterateColumn(M1, 3, 'ln(abs(x))');
     Write_ComplexMatrix(M1);
     Write_Message('');

     ComplexMatrix_IterateColumn(M1, 4, 'sqr(x)');
     Write_ComplexMatrix(M1);
     Write_Message('');

     ComplexMatrix_IterateColumn(M1, 5, '666');
     Write_ComplexMatrix(M1);
     Write_Message('');

     finally
     DestroyComplexMatrix(M1);
     ShowMessage('Memory consumed: ' + IntToStr(CurrentMatrixMemoryUse));
     end;
end;

{ Use a matrix and the function parser to hold and calculate several values.
  As an example, we will draw 2 mathematical functions.                         }
procedure TForm1.Draw1Click(Sender: TObject);
var M1: RealMatrix;
    i, j: integer;
begin
     InitRealMatrix(M1,1, 1);
     PaintBox1.Visible:= true;
     Memo1.Visible:= false;

     try
     Redimension_RealMatrix(M1, 2, PaintBox1.Width);

     ShowMessage('Initializing matrix. Please Wait ...');

     // Fill in matrix elements
     for j:= 1 to M1.Col do
     begin
     M1.Matrix^[1]^[j]:= j - (PaintBox1.Width div 2);
     M1.Matrix^[2]^[j]:= j - (PaintBox1.Width div 2);
     end;

     // Apply functions on elements
     RealMatrix_IterateRow(M1, 1, 'sqr(x)');
     RealMatrix_IterateRow(M1, 2, 'abs(x)');

     ShowMessage('Matrix inititialization done. Now going to draw functions ...');

     // Draw axes on paintbox
     PaintBox1.Canvas.Pen.Color:= clBlue;
     PaintBox1.Canvas.Moveto(0, PaintBox1.Height div 2);
     PaintBox1.Canvas.Lineto(PaintBox1.Width, PaintBox1.Height div 2);
     PaintBox1.Canvas.Moveto(PaintBox1.Width div 2, 0);
     PaintBox1.Canvas.Lineto(PaintBox1.Width div 2, PaintBox1.Height);

     // Draw functions
     for j:= 1 to M1.Col do
     begin
     PaintBox1.Canvas.Pixels[j, PaintBox1.Height div 2 - Round(M1.Matrix^[1]^[j])]:= clRed;
     PaintBox1.Canvas.Pixels[j, PaintBox1.Height div 2 - Round(M1.Matrix^[2]^[j])]:= clGreen;
     end;

     finally
     DestroyRealMatrix(M1);
     end;
end;

end.
