{ Demo program for ChroMathsome - Linear Algebra.

  As an example, we will save and reload a bitmap picture.
  The picture will be stored using a matrix and a vector.
  Note:
  In this demonstration I've omitted to do any compressing of the file when it is saved.
  The file I/O functions in ChroMathsome - Linear Algebra do not use any kind of compression
  (maybe in a next version it will - any hints/ideas/tips on how to do the compression? Let me know!).
  Maybe some smart guys out there can take this code as a starting point to create a "new"
  file saving algorithm.

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: June 30, 1998
  Reader may do with this code whatever he/she wants (give credits where credits are due ...).

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "O".              }
unit Graphs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LinAlg, Menus, ExtCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    File1: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    procedure Close1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    function ColorPosInVector(Vec: RealVector; Cl: longint): integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  V: RealVector;

implementation

{$R *.DFM}

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

function TForm1.ColorPosInVector(Vec: RealVector; Cl: longint): integer;
var i: longint;
    match: boolean;
begin
     match:= false;
     i:= 0;
     repeat
     begin
     i:= i + 1;
     if Round(Vec.Vector^[i]) = Cl then match:= true;
     end;
     until ((i = Vec.Dimension) or (match = true));
     if match = true then ColorPosInVector:= i
     else ColorPosInVector:= 0;
end;

{ Write the picture to a file.
  The colors are stored in a vector (V) while the matrix elements represent the dots of the picture.
  Every matrix element has a value that represents the position of the respective color of the
  respective dot in V.
  This works quite well for 16 color images.
  256, 64k, ..., colors -> Try to improve the code.                                           }
procedure TForm1.Save1Click(Sender: TObject);
var M: RealMatrix;
    i, j, k, first: integer;
    LColor: TColor;
    Li: longint;
begin
     InitRealMatrix(M, 1, 1);

     try
     first:= 0;  // V is initialized to have a dimension of 1.
     Redimension_RealMatrix(M, Image1.Width, Image1.Height);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     { Store the colors of the picture. }
     begin
       LColor:= Image1.Canvas.Pixels[i, j];
       Li:= ColorToRGB(LColor);

       if first = 0 then
       begin
         V.Vector^[1]:= Li;
         k:= 0;
         first:= 1
       end
       else
       begin // If V already contains the color ...
        k:= ColorPosInVector(V, Li);
        if k = 0 then RealVector_AddElement(V, Li); // If not -> add it!
       end;
       if k > 0 then M.Matrix^[i]^[j]:= k
       else M.Matrix^[i]^[j]:= V.Dimension;
     end;

     SaveRealMatrixToFile(M, 'DUMMYM.TXT');
     SaveRealVectorToFile(V, 'DUMMYV.TXT');

     finally
     DestroyRealMatrix(M);
     end;
end;

{ Load (= read) the picture from a file and reconstruct the bitmap.           }
procedure TForm1.Open1Click(Sender: TObject);
var M: RealMatrix;
    i, j, k: integer;
    Li: longint;
begin
     if FileExists('DUMMYM.TXT') = false then
     MessageDlg('You must save the image first!', mtError, [mbOK], 0)
     else
     begin
     InitRealMatrix(M, 1, 1);

     try

     LoadRealMatrixFromFile(M, 'DUMMYM.TXT'); // Updates dimensions of M if needed!
     LoadRealVectorFromFile(V, 'DUMMYV.TXT'); // Updates dimension of V if needed!

     { Draw the image on Form1's Canvas. }
     for i:= 1 to M.Row do
     for j:= 1 to M.Col do
     begin
       k:= Round(M.Matrix^[i]^[j]); // The values of M and V are of type extended ...
       Form1.Canvas.Pixels[50 + i, 200 + j]:= Round(V.Vector^[k]);
     end;

     finally
     DestroyRealMatrix(M);
     end;
     end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     InitRealVector(V, 1);
     V.Vector^[1]:= -1;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     DestroyRealVector(V);
end;

end.
